﻿"""
Created on 2019年6月28日
@author: 刘益宗
系统常量配置
"""
import enum


class UpgradeType:
    Full = 0
    Part = 1

    All_List = [Part, Full]


class ModelConfKey:
    """
    模块配置key，与数据库modelConf表配合使用
    """
    LanguageMode = 'LanguageMode'  # 语言模式
    ScreenLock = 'ScreenLock'  # 屏幕锁定
    LayerLock = 'LayerLock'  # 图层锁定
    RealtimeDisplay = 'RealtimeDisplay'  # 是否实时上屏
    PresetPlay = 'PresetPlay'
    VideoStatus = 'VideoStatus'
    NeutralMode = 'NeutralMode'  # 中性模式
    PresetShowId = 'PresetShowId'  # 是否展示场景Id


class UploadFileType:
    """
    上传文件类型
    """
    BKG = 1
    OSD = 2
    Device_Settings = 3
    Upgrade = 4
    Input_EDID = 5
    NomarkMode = 6
    Normal = 7
    Font = 8
    IPC = 9
    Sdp = 10


class LanguageMode:
    """
    语言模式
    """
    English = 0
    Chinese = 1


class BKGFile:
    """
    bkg图片文件
    """
    # 最大图片数量
    Max_BKG_Count = 16

    # 图片大小限制
    bkg_limit = [
        {'width': 8192, 'height': 1024},
        {'width': 4096, 'height': 2048}
    ]

    def __init__(self, width, height):
        self.width = width
        self.height = height

    def valid_pic(self):
        """
        判断图片是否合法
        :return: True/False
        """
        for limit in self.bkg_limit:
            if self.width <= limit['width'] and self.height <= limit['height']:
                return True
        return False


class InterfaceType:
    """
    interfaceType 对应值
    """
    EXP = 1
    Single_Link_DVI = 2
    Dual_Link_DVI = 3
    HDMI13 = 4
    HDMI14 = 5
    HDMI20 = 6
    DP11 = 7
    DP12 = 8
    SDI_3G = 9
    VGA = 10
    CVBS = 11
    YpbPr = 12
    RJ45 = 13
    USB = 14
    HDBaseT = 15
    HDBaseT_4K = 16
    OpticalFiber = 17
    SDI_12G = 18
    DB9_MALE = 19
    DB9_FEMALE = 20
    IO = 21
    RELAY = 22
    IR = 23
    Phoenix = 24
    SDI_3G_OUT = 25
    RJ45DL = 26
    Optical2x_Fiber = 27
    # 允许设置EDID接口列表
    _EDID_Setting_List = [Single_Link_DVI, Dual_Link_DVI, HDMI13, HDMI14, HDMI20, DP11, DP12, OpticalFiber, HDBaseT, DB9_MALE, SDI_3G_OUT, RJ45, RJ45DL, SDI_12G]
    # 允许功能切换的接口列表
    _Allow_Function_Change_List = [Single_Link_DVI, Dual_Link_DVI, HDMI13, HDMI14, HDMI20, DP12, HDBaseT, DB9_MALE, DB9_FEMALE, RJ45, RJ45DL]

    @staticmethod
    def allow_edid_setting(interface_type, additional_param=None):
        """
        是否允许设置EDID
        :param interface_type: 接口类型
        :return:  1：允许，0: 不允许
        """
        if interface_type in InterfaceType._EDID_Setting_List:
            if additional_param is None:
                return 1
            else:
                return 0
        return 0


class CardType:
    """
    卡类型
    """
    NoCard = 0
    In = 1
    Out = 2
    Sender = 3
    Mvr = 4
    STD_IO_Card = 5


class ConnectCapacity:
    """
    链接容量，默认使用SL
    """
    SL = 0
    DL = 1
    K4 = 2

    Default = SL


class LayerType:
    """
    开窗时图层类型图层类型
    """
    # ,0：mix，1：SL , F8Plus使用1
    mix = 0
    SL = 1
    Default = SL


class SourceType:
    NoSource = 0  # 无源,
    InputSource = 1  # 输入类型,
    IPCSource = 3  # IPC类型
    Color = 4  # 纯色
    IPCMontage = 5  #
    IPC = 255
    NdiSource = 8 # Ndi源



class DeviceType:
    Unknown = 0
    ES = 1
    HS = 2
    H_9U = 0x750A  # H_9U
    H_5U = 0x750B  # H_Ctrl_5U
    H_2U = 0x750D  # H_Ctrl_2U
    H_15U = 0x750E  # H_Ctrl_15U
    H_15U_Pro = 0x7510
    H_20U = 0x751E # H_Ctrl_20U

    P_H_9U = 0x7525  # H_9U
    P_H_5U = 0x7524  # H_Ctrl_5U
    P_H_2U = 0x7523  # H_Ctrl_2U
    P_H_15U = 0x752C  # H_Ctrl_15U
    P_H_15U_PRO = 0x752D  # H_Ctrl_15UPRO

    # E系列
    Es_3 = 0x750F
    Es_7 = 0x7511

    Alita_9 = 0x751C
    Alita_15 = 0x751D
    Alita_2 = 0x7521
    Alita_5 = 0x7522
    Alita_20 = 0x7530


    ES_ALL = [Es_3, Es_7]
    HS_ALL = [H_9U, H_5U, H_2U, H_15U, H_15U_Pro, H_20U]
    Alita_ALL = [Alita_9, Alita_15, Alita_2, Alita_5, Alita_20]
    P_Main_All = [P_H_9U, P_H_5U, P_H_2U, P_H_15U, P_H_15U_PRO]


class UpgradeConfig:
    H_160T_HardVer = '3.0'
    H_160T_FPGA_Filename = 'FPGA_160T.dat'
    H_RK_MCU_Filename = 'MCU_RK.dat'
    H_RK_FPGA_Filename = 'FPGA_RK.dat'
    H_Cost_Reduction__FPGA_Filename = 'FPGA_1.dat'
    H_Cost_Reduction_MCU_Filename = 'MCU_1.dat'




class ModelId:
    """
    卡的ModelId
    """
    H_AUX_2U = 0x7B82
    Input_Mother_Child_Card = 0x7B00
    Input_4_3G_SDI_Card = 0x7B01
    Input_2_12G_SDI_Card = 0x7B02
    Input_HDMI20_DP12 = 0x7B03
    Input_2_DP11_Card = 0x7B04
    Input_IPC_Card = 0x7B05  # Input_2RJ45_1USB_Card = 0x7B05
    Input_4_HDBaseT_Card = 0x7B06
    H_2xHDMI20_I = 0x7B07
    H_2xHDMI20_2xDP12_I = 0x7B09  # 4选2输入卡
    H_2x_Fiber_I = 0x7B10  # 2路光纤
    H_1xST2110_I = 0x7B11  # 2110 卡

    H_4xHDBaseT_Card = 0x7B0A # H4路 HDBaseT 输入卡
    H_2xAudio_I_O = 0x7B0C #H2路音频输入2路音频输出卡
    H_2xAudio_I_Dante_Card = 0x7B0D #H_2路音频输入2路音频输出2路Dante卡
    H_4xHDMI_Audio_I = 0x7B0E  # 四路HDMI随路音频输入卡
    H_2xHDMI_DP_I = 0x7B0F  # 四选二随路音频输入卡

    Output_4_DVI_Card = 0x7B40
    Output_4_HDMI_Card = 0x7B41
    Output_4_DP11_Card = 0x7B42
    Output_2_RJ45_Card = 0x7B44
    Output_Two_In_One_Card = 0x7B46
    Output_2RJ45_1HDMI_Card = 0x7B47
    Output_Two_In_One_Card_20 = 0x7B48
    Output_2_HDMI20_Card = 0x7B49 # 4k

    Output_4_Fiber_Card = 0x7B4A  # 4光纤卡
    Output_4_RJ45_Card =  0x7B4B # H_4路网口 视频输出卡
    Output_4_3G_SDI_Card = 0x7B4C  # H_4路 3G-SDI 输出卡
    Output_12_3G_SDI_Card = 0x7B4D  # H1路 12G-SDI 输出卡
    Output_4_HDBaseT_Card = 0x7B4E  # H_4路BaseT 视频输出卡

    Not_Online = 0x7501

    Pd_Input_Mother_Child_Card = 0x9000
    Pd_Input_4_3G_SDI_Card = 0x9001
    Pd_Input_2_12G_SDI_Card = 0x9002
    Pd_Input_HDMI20_DP12 = 0x9003
    Pd_Input_2_DP11_Card = 0x9004
    Pd_H_2xHDMI20_I = 0x9007
    Pd_H_2xHDMI20_2xDP12_I = 0x9009  # 4选2输入卡

    Pd_H_4xHDMI_Audio_I = 0x900E  # 四路HDMI随路音频输入卡
    Pd_H_2xHDMI_DP_I = 0x900F  # 四选二随路音频输入卡

    Pd_Output_4_DVI_Card = 0x9040
    Pd_Output_4_HDMI_Card = 0x9041
    Pd_Output_Two_In_One_Card = 0x9046
    Pd_Output_Two_In_One_Card_20 = 0x9048
    Pd_Output_2_HDMI20_Card = 0x9049  # 4k
    Pd_Output_4_RJ45_Card = 0x904B  # H_4路网口 视频输出卡

    Two_In_One_Card_List = [Output_Two_In_One_Card, Output_Two_In_One_Card_20, Output_4_Fiber_Card, Pd_Output_Two_In_One_Card, Pd_Output_Two_In_One_Card_20]
    DL_Output_Card_List = [Output_4_DVI_Card, Output_4_HDMI_Card, Output_4_HDBaseT_Card, Output_4_RJ45_Card, Pd_Output_4_DVI_Card, Pd_Output_4_HDMI_Card, Pd_Output_4_RJ45_Card]
    Input_SL_DL_List = [Input_Mother_Child_Card, H_4xHDBaseT_Card, H_4xHDMI_Audio_I, Pd_Input_Mother_Child_Card, Pd_H_4xHDMI_Audio_I]
    H_160T_Card_ModelId_List = [Input_Mother_Child_Card, Input_2_DP11_Card, H_2xHDMI20_I, Input_HDMI20_DP12, H_AUX_2U,
                                H_2xHDMI20_2xDP12_I, Pd_Input_HDMI20_DP12, Pd_H_2xHDMI20_I, Pd_H_2xHDMI20_2xDP12_I, Pd_Input_Mother_Child_Card, Pd_Input_2_DP11_Card]
    H_Cost_Reduction_Card_ModelId_List =[Output_4_HDMI_Card, Output_4_DVI_Card, Pd_Output_4_DVI_Card, Pd_Output_4_HDMI_Card]


class CardName:
    Unknown = 'unKnown'
    @staticmethod
    def get_card_name(model_id):
        card_name = CardName.Unknown
        if model_id == ModelId.H_AUX_2U:
            card_name = 'H_AUX_2U'
        elif model_id == ModelId.Input_Mother_Child_Card:
            card_name = 'Input_Mother_Child_Card'
        elif model_id == ModelId.Input_4_3G_SDI_Card:
            card_name = 'Input_4_3G_SDI_Card'
        elif model_id == ModelId.Input_2_12G_SDI_Card:
            card_name = 'Input_2_12G_SDI_Card'
        elif model_id == ModelId.Input_HDMI20_DP12:
            card_name = 'Input_HDMI20_DP12'
        elif model_id == ModelId.Input_2_DP11_Card :
            card_name = 'Input_2_DP11_Card'
        elif model_id == ModelId.Input_IPC_Card:
            card_name = 'Input_IPC_Card'
        elif model_id == ModelId.Input_4_HDBaseT_Card:
            card_name = 'Input_4_HDBaseT_Card'
        elif model_id == ModelId.H_2xHDMI20_I:
            card_name = 'H_2xHDMI20_I'
        elif model_id == ModelId.H_2xHDMI20_2xDP12_I:
            card_name = 'H_2xHDMI20_2xDP12_I'
        elif model_id == ModelId.Output_4_DVI_Card:
            card_name = 'Output_4_DVI_Card'
        elif model_id == ModelId.Output_4_HDMI_Card:
            card_name = 'Output_4_HDMI_Card'
        elif model_id == ModelId.Output_4_DP11_Card:
            card_name = 'Output_4_DP11_Card'
        elif model_id == ModelId.Output_4_HDBaseT_Card:
            card_name = 'Output_4_HDBaseT_Card'
        elif model_id == ModelId.Output_2_RJ45_Card:
            card_name = 'Output_2_RJ45_Card'
        elif model_id == ModelId.Output_4_Fiber_Card:
            card_name = 'Output_4_Fiber_Card'
        elif model_id == ModelId.Output_Two_In_One_Card:
            card_name = 'Output_Two_In_One_Card'
        elif model_id == ModelId.Output_2_HDMI20_Card:
            card_name = 'Output_2_HDMI20_Card'
        elif model_id == ModelId.Not_Online:
            card_name = 'Not_Online'
        elif model_id == ModelId.Output_4_RJ45_Card:
            card_name = 'Output_4_RJ45_Card'
        elif model_id == ModelId.Output_4_3G_SDI_Card:
            card_name = 'Output_4_3G_SDI_Card'
        elif model_id == ModelId.Output_12_3G_SDI_Card:
            card_name = 'Output_12_3G_SDI_Card'
        elif model_id == ModelId.H_4xHDBaseT_Card:
            card_name = 'H_4xHDBaseT_Card'
        elif model_id == ModelId.H_1RJ45_I:
            card_name = 'H_1RJ45_I'
        elif model_id == ModelId.H_2xAudio_I_O:
            card_name = 'H_2xAudio_I_O'
        elif model_id == ModelId.H_2xAudio_I_Dante_Card:
            card_name = 'H_2xAudio_I_Dante_Card'
        elif model_id == ModelId.H_4xHDMI_Audio_I:
            card_name = 'H_4xHDMI_Audio_I'
        elif model_id == ModelId.H_2xHDMI_DP_I:
            card_name = 'H_2xHDMI_DP_I'
        return card_name


class CardCategory:
    Unknown = 0
    TwoInOneCard = 1
    MotherChildCard = 2
    IPCCard = 3
    K4_HDMI_DP_Card = 4
    TwoInOneCard_20 = 5
    Output4DVICard = 6
    Output4HDMICard = 7
    Output4KCard = 8
    StdIOCard = 9
    Input2x4K = 10
    H_2xHDMI20_2xDP12_I = 11  # 4选2或者 单DP
    H_Output4LinkCard = 12 # 4网口卡
    Output4FiberCard = 15  # 4光纤卡
    Input4xHDBaseTCard = 16  #
    AudioCard = 17  #
    SDI3GCard = 18  #
    H4xHDMIAudioCard = 19  #
    Output4HDBaseTCard = 20
    SDI12GCard = 21
    H2xHDMIDPICard = 22
    H2xFIBERCard = 23
    H1xST2110Card = 24



    @staticmethod
    def get_card_category(model_id):
        card_category = CardCategory.Unknown
        if model_id == ModelId.Input_Mother_Child_Card:
            card_category = CardCategory.MotherChildCard
        elif model_id == ModelId.Pd_Input_Mother_Child_Card:
            card_category = CardCategory.MotherChildCard
        elif model_id == ModelId.Output_Two_In_One_Card:
            card_category = CardCategory.TwoInOneCard
        elif model_id == ModelId.Pd_Output_Two_In_One_Card:
            card_category = CardCategory.TwoInOneCard
        elif model_id == ModelId.Output_Two_In_One_Card_20:
            card_category = CardCategory.TwoInOneCard_20
        elif model_id == ModelId.Pd_Output_Two_In_One_Card_20:
            card_category = CardCategory.TwoInOneCard_20
        elif model_id == ModelId.Input_IPC_Card:
            card_category = CardCategory.IPCCard
        elif model_id == ModelId.Input_HDMI20_DP12:
            card_category = CardCategory.K4_HDMI_DP_Card
        elif model_id == ModelId.Pd_Input_HDMI20_DP12:
            card_category = CardCategory.K4_HDMI_DP_Card
        elif model_id == ModelId.Output_4_DVI_Card:
            card_category = CardCategory.Output4DVICard
        elif model_id == ModelId.Pd_Output_4_DVI_Card:
            card_category = CardCategory.Output4DVICard
        elif model_id == ModelId.Output_4_HDMI_Card:
            card_category = CardCategory.Output4HDMICard
        elif model_id == ModelId.Pd_Output_4_HDMI_Card:
            card_category = CardCategory.Output4HDMICard
        elif model_id == ModelId.Output_2_HDMI20_Card:
            card_category = CardCategory.Output4KCard
        elif model_id == ModelId.H_2xHDMI20_I:
            card_category = CardCategory.Input2x4K
        elif model_id == ModelId.Pd_H_2xHDMI20_I:
            card_category = CardCategory.Input2x4K
        elif model_id == ModelId.H_2xHDMI20_2xDP12_I:
            card_category = CardCategory.H_2xHDMI20_2xDP12_I
        elif model_id == ModelId.Pd_H_2xHDMI20_2xDP12_I:
            card_category = CardCategory.H_2xHDMI20_2xDP12_I
        elif model_id == ModelId.Output_4_Fiber_Card:
            card_category = CardCategory.Output4FiberCard
        elif model_id == ModelId.H_4xHDBaseT_Card:
            card_category = CardCategory.Input4xHDBaseTCard
        elif model_id == ModelId.H_2xAudio_I_O:
            card_category = CardCategory.AudioCard
        elif model_id == ModelId.Output_4_3G_SDI_Card:
            card_category = CardCategory.SDI3GCard
        elif model_id == ModelId.Output_12_3G_SDI_Card:
            card_category = CardCategory.SDI12GCard
        elif model_id == ModelId.Output_4_HDBaseT_Card:
            card_category = CardCategory.Output4HDBaseTCard
        elif model_id == ModelId.H_4xHDMI_Audio_I:
            card_category = CardCategory.H4xHDMIAudioCard
        elif model_id == ModelId.Pd_H_4xHDMI_Audio_I:
            card_category = CardCategory.H4xHDMIAudioCard
        elif model_id == ModelId.H_2xHDMI_DP_I:
            card_category = CardCategory.H2xHDMIDPICard
        elif model_id == ModelId.Pd_H_2xHDMI_DP_I:
            card_category = CardCategory.H2xHDMIDPICard
        elif model_id == ModelId.Output_4_RJ45_Card:
            card_category = CardCategory.H_Output4LinkCard
        elif model_id == ModelId.Pd_Output_4_RJ45_Card:
            card_category = CardCategory.H_Output4LinkCard
        elif model_id == ModelId.H_2x_Fiber_I:
            card_category = CardCategory.H2xFIBERCard
        elif model_id == ModelId.H_1xST2110_I:
            card_category = CardCategory.H1xST2110Card
        elif model_id == ModelId.Pd_Output_2_HDMI20_Card:
            card_category = CardCategory.Output4KCard

        return card_category


class FunctionType:
    SL = 0
    DL = 1
    HDMI20 = 2
    DP12 = 3
    Disable = 0xFF


class UpgradeItem(enum.Enum):
    """
    升级项目枚举
    """
    MCU = 1
    MCU_A = 2
    MCU_B = 3
    Aux_MCU = 4
    IMAGE = 5
    FNOT = 6
    FPGA = 7
    FPGA_A = 8
    FPGA_B = 9
    AuxFPGA_A = 10
    AuxFPGA_B = 11
    FPGA_A_RE = 12
    FPGA_B_RE = 13
    FPGA_RE = 21
    FPGA_COST = 23



UpgradeFileModel = {
    0x750A: 'H_Ctrl',  # H_Ctrl_9U
    0x750B: 'H_Ctrl',  # H_Ctrl_5U
    0x750D: 'H_Ctrl',  # H_Ctrl_2U
    0x750E: 'H_Ctrl',  # H_Ctrl_15U
    0x7510: 'H_Ctrl',  # H_Ctrl_15U_Pro
    0x751E: 'H_Ctrl',  # H_Ctrl_20U

    0x751C: 'Alita_Ctrl',
    0x751D: 'Alita_Ctrl',

    0x7521: 'Alita_Ctrl',
    0x7522: 'Alita_Ctrl',
    0x7530: 'Alita_Ctrl',

    # 平达系列
    0x7525: 'H_CTM_Ctrl',  # H_Ctrl_9U
    0x7524: 'H_CTM_Ctrl',  # H_Ctrl_5U
    0x7523: 'H_CTM_Ctrl',  # H_Ctrl_2U
    0x752C: 'H_CTM_Ctrl',  # H_Ctrl_15U
    0x752D: 'H_CTM_Ctrl',  # H_Ctrl_15U_PRO

    0x7B47: 'H_MVR',

    0x7B80: 'H_AUX_9U',
    0x7B81: 'H_AUX_5U',
    0x7B82: 'H_AUX_2U',
    0x7B83: 'H_AUX_15U',
    0x7B84: 'H_BACK_15U',
    0x7B85: 'H_AUX_15U_PRO',
    0x7B86: 'H_BACK_15U_PRO',
    0x7B87: 'H_AUX_20U',
    0x7B88: 'H_BACK_20U',

    0x7B00: 'H_4xSLDVI_I',
    0x7B01: 'H_4x3GSDI_I',
    0x7B02: 'H_12GSDI_I',
    0x7B03: 'H_HDMI20_DP12_I',
    0x7B04: 'H_2xDP11_I',
    0x7B05: 'H_2xRJ45_USB_I',
    0x7B06: 'H_4xHDBase_T_I',
    0x7B07: 'H_2xHDMI20_I',
    0x7B09: 'H_2xHDMI20_2xDP12_I', # 4选2
    0x7B10: 'H_2xFiber_I',  # 2路光纤输入卡
    0x7B11: 'H_1xST2110_I',# 2110 卡
    0x7B0A: 'H_4xHDBaseT_I',
    0x7B0C: 'H_AUDIO_I', # 音频卡
    0x7B0E: 'H_4xHDMI_AUDIO_I',  # 四路HDMI随路音频输入卡
    0x7B0F: 'H_2xHDMI20_2xDP12_AUDIO_I',  # 四选二随路音频输入卡


    0x7B40: 'H_4xDVI_O',
    0x7B41: 'H_4xHDMI_O',
    0x7B42: 'H_4xDP11_O',
    0x7B43: 'H_4xHDBaseT',
    0x7B44: 'H_2xRJ45_O',
    0x7B4A: 'H_4xFiber_O', # 4光纤卡
    0x7B4B: 'H_4xRJ45_O', # 4网口卡
    0x7B46: 'H_16xRJ_2xFib_O',
    0x7B48: ('H_20xRJ_O', 'H_20xRJ45_O'),
    0x7B49: 'H_2xHDMI20_O',
    0x7B4C: 'H_4x3GSDI_O',  # 3GSDI输出卡
    0x7B4D: 'H_1x12GSDI_O',# 12GSDI输出卡
    0x7B4E: 'H_4xHDBaseT_O', # HDBaset输出卡

    # 平达系列
    0x9000: 'H_CTM_4xSLDVI_I',

    0x9001: 'H_CTM_4x3GSDI_I',

    0x9002: 'H_CTM_12GSDI_I',
    0x9003: 'H_CTM_HDMI20_DP12_I',
    0x9004: 'H_CTM_2xDP11_I',

    0x9007: 'H_CTM_2xHDMI20_I',
    0x9009: 'H_CTM_2xHDMI20_2xDP12_I',  # 4选2

    0x900E: 'H_CTM_4xHDMI_AUDIO_I',  # 四路HDMI随路音频输入卡
    0x900F: 'H_CTM_2xHDMI20_2xDP12_AUDIO_I',  # 四选二随路音频输入卡

    0x9040: 'H_CTM_4xDVI_O',
    0x9041: 'H_CTM_4xHDMI_O',
    0x9046: 'H_CTM_16xRJ_2xFib_O',
    0x9047: 'H_CTM_MVR',
    0x9048: 'H_CTM_20xRJ45_O',
    0x9049: 'H_CTM_2xHDMI20_O',
    0x904B: 'H_CTM_4xRJ45_O', # 网口输出卡

}

# H9 aux 替代料
class UpgradeH9AuxReplaceDataConfig:
    """
    硬件版本1.0 升级MCU.dat(1)、FPGA_A.dat(8)、FPGA_B.dat(9)
    硬件版本2.0 升级MCU.dat(1)、FPGA_A_1.dat(12)、FPGA_B_1.dat(13)

    2.RK IPC卡
    "chipBrief": RK3566 升级MCU_RK.dat(2)、FPGA_RK.dat(7)
    "chipBrief": HI3531 升级MCU.dat(1)、FPGA.dat(7)
    """
    DeviceModelIdList = [DeviceType.H_9U, DeviceType.Alita_9, DeviceType.P_H_9U]  # 0x750A
    HardWareVersion = ['2.0']
    HardWareVersionMapping = {
        '1.0': {
            'MCU': {
                'file': 'MCU.dat',
                'code': 1
            },
            'FPGA_A': {
                'file': 'FPGA_A.dat',
                'code': 8
            },
            'FPGA_B': {
                'file': 'FPGA_B.dat',
                'code': 9
            },
        },
        '2.0': {
            'MCU': {
                'file': 'MCU.dat',
                'code': 1
            },
            'FPGA_A': {
                'file': 'FPGA_A_1.dat',
                'code': 12
            },
            'FPGA_B': {
                'file': 'FPGA_B_1.dat',
                'code': 13
            },
        }

    }


# 4k输出
class Upgrade2HDMI20OutputDataConfig:
    HardWareVersion = ['2.0']
    HardWareVersionNum = 4
    dict_key = 'version'
    HardWareVersionMapping = {
        '1.0': {
            'MCU': {
                'file': 'MCU.dat',
                'code': 1
            },
            'FPGA': {
                'file': 'FPGA.dat',
                'code': 7
            }
        },
        '2.0': {
            'FPGA': {
                'file': 'FPGA_1.dat',
                'code': 21
            },
        },
        dict_key: {
            'MCU': {
                'file': 'MCU_1.dat',
                'code': 2
            },
            'FPGA': {
                'file': 'FPGA_1.dat',
                'code': 23
            }
        }

    }


class TestPatternEnum(enum.Enum):
    """
    测试画面枚举
    """
    Black = 0x0000  # 黑色
    Red = 0x0001  # 红色
    Green = 0x0002  # 绿色
    Blue = 0x0003  # 蓝色
    White = 0x0004  # 白色
    Close = 0xFFFF  # 关闭
    # 0x0005    竖条
    # 0x0006    横条
    # 0x0007    棋盘
    # 0x0100    水平红色渐变
    # 0x0101    水平绿色渐变
    # 0x0102    水平蓝色渐变
    # 0x0103    水平白色渐变
    # 0x0104    垂直红色渐变
    # 0x0105    垂直绿色渐变
    # 0x0106    垂直蓝色渐变
    # 0x0107    垂直白色渐变
    # 0x0200    白色横线
    # 0x0201    白色竖线
    # 0x0202    白色左斜
    # 0x0203    白色右斜
    # 0x0204    横竖线交叉
    # 0x0205    左斜右斜交叉
    # 0x0300    定位
    # 0xFFFE    自定义测试画面
    #


class ScreenDefaultValue:
    """
    屏幕参数默认值
    """
    ImageQuality = {
        "contrast": {
            "all": 50,
            "R": 50,
            "G": 50,
            "B": 50
        },
        "brightness": {
            "all": 50,
            "R": 50,
            "G": 50,
            "B": 50
        },
        "hue": 0,
        "saturation": 50
    }
    Gamma = 1
    Test_Pattern = {
        "testPattern": TestPatternEnum.Close.value,
        "bright": 2,
        "grid": 3,
        "speed": 2
    }


class UserType:
    SysUser = 0
    LicenseUser = 1


class IPCDefaultValue:
    """ipc源路数"""
    Slot_Max_Source_Count = 512
    Slot_Max_Channel_Count = 1024
    Source_Max_Channel_Count = 512
    Source_Channel_Create_Default_Count = 50
    Batch_Create_Default_Count = 50
    Group_Max_Source_Count = 1024
    Group_Max_Count = 50
    Export_Source_Count = 100
    Group_Max_Add_Source_Count = 1000

class RoleMaxCountValue:
    Role_Max_Count = 99
    Role_Root_Id = 1
    Role_Default_Id = 2


class TwoInOneCardData_1040:
    """
    二合一卡1040万规格
    """
    TimingMaxBandWidth_1 = 165000000
    TimingMaxBandWidth_2 = 165000000
    SlotMaxArea = 624600000
    SlotMaxWidth = 10240
    SlotMaxHeigth = 10240
    SlotMinWidth = 800
    SlotMinHeight = 600

    # 二合一卡单个接口最大带载
    SingleOutputDefaultResolution = 1920 * 1080 * 60

    # 接口标准宽度
    SingleOutputStandardWidth = 2560
    # 二合一卡接口最大宽度
    SingleOutputMaxWidth = 2560


class FourFiberCardData:
    """
    四光纤卡独立模式规格
    """
    TimingMaxBandWidth_1 = 330000000
    TimingMaxBandWidth_2 = 330000000
    SlotMaxArea = 1248600000
    SlotMaxWidth = 16384
    SlotMaxHeigth = 16384
    SlotMinWidth = 800
    SlotMinHeight = 600

    # 四光纤卡单个接口最大带载
    SingleOutputDefaultResolution = 1920 * 1080 * 60

    # 接口标准宽度
    SingleOutputStandardWidth = 2560
    # 四光纤卡接口最大宽度
    SingleOutputMaxWidth = 2560


class FourFiberCardCopyData:
    """
    四光纤卡复制模式规格
    """
    TimingMaxBandWidth_1 = 330000000
    TimingMaxBandWidth_2 = 330000000
    SlotMaxArea = 624300000
    SlotMaxWidth = 16384
    SlotMaxHeigth = 16384
    SlotMinWidth = 800
    SlotMinHeight = 600

    # 四光纤卡单个接口最大带载
    SingleOutputDefaultResolution = 1920 * 1080 * 60

    # 接口标准宽度
    SingleOutputStandardWidth = 2560
    # 四光纤卡接口最大宽度
    SingleOutputMaxWidth = 2560

class FourFiberCardBackupData:
    """
    四光纤卡备份模式规格
    """
    TimingMaxBandWidth_1 = 330000000
    TimingMaxBandWidth_2 = 330000000
    SlotMaxArea = 624300000
    SlotMaxWidth = 16384
    SlotMaxHeigth = 16384
    SlotMinWidth = 800
    SlotMinHeight = 600

    # 四光纤卡单个接口最大带载
    SingleOutputDefaultResolution = 1920 * 1080 * 60

    # 接口标准宽度
    SingleOutputStandardWidth = 2560
    # 四光纤卡接口最大宽度
    SingleOutputMaxWidth = 2560


class TwoInOneCardData_1300:
    """
    二合一卡1300万规格
    """
    TimingMaxBandWidth_1 = 208000000
    TimingMaxBandWidth_2 = 165000000
    SlotMaxArea = 13120000 * 60
    SlotMaxWidth = 10752
    SlotMaxHeigth = 10752
    SlotMinWidth = 800
    SlotMinHeight = 600

    # 二合一卡单个接口最大带载
    SingleOutputDefaultResolution = 1920 * 1080 * 60

    # 接口标准宽度
    SingleOutputStandardWidth = 2560
    # 二合一卡接口最大宽度
    SingleOutputMaxWidth = 2560

    TimingMaxBandWidth = 165000000


class Output4KCardData:
    TimingMaxBandWidth = 594000000
    SingleOutputTimingMaxBandWidth = 150000000


class InputData:
    Group_Max_Count = 50


Output_DL_Timing_Default = {
    "horizontal": {"addrTime": 1920, "frontPorch": 88, "sync": 44, "syncPolarity": 0, "totalTime": 2200},
    "vertical": {"addrTime": 2160, "frontPorch": 8, "sync": 10, "syncPolarity": 0, "totalTime": 2250},
    "refreshRate": 30}


Output_SL_Timing_Default = {
    "horizontal": {"addrTime": 1920, "frontPorch": 88, "sync": 44, "syncPolarity": 0, "totalTime": 2200},
    "vertical": {"addrTime": 1080, "frontPorch": 4, "sync": 5, "syncPolarity": 0, "totalTime": 1125},
    "refreshRate": 60}


Max_Band_Capacity_Map = {

    "input": {
        ConnectCapacity.SL: 165000000,
        ConnectCapacity.DL: 297000000,
        ConnectCapacity.K4: 594000000
    },
    "output": {
        ConnectCapacity.SL: 165000000,
        ConnectCapacity.DL: 297000000,
        ConnectCapacity.K4: 594000000
    }
}


# Output_DL_Timing_Default = {
#     "horizontal": {"addrTime": 3840, "frontPorch": 176, "sync": 88, "syncPolarity": 0, "totalTime": 4400},
#     "vertical": {"addrTime": 1080, "frontPorch": 4, "sync": 5, "syncPolarity": 0, "totalTime": 1125},
#     "refreshRate": 60}

class MVRWindowType:
    Input = 0
    PGM = 1
    PVW = 2
    IPC = 3


class UpgradeCardType:
    RK = "RK3566"
    HS = "HI3531"


class UpgradeCardCode:
    HS_Code = 1
    Rk_Code = 2


class DataType:
    Menu_Type = 1
    Input_Type = 2
    Output_Type  = 3
    Screen_Type = 4
    Mvr_Type = 5
    User_Type = 6
    Role_Type = 7


# 权限功能菜单code值
class PermissionCode:
    Screen_Configuration = 1
    Screen_Edit = 2
    Pre_Supervision = 3
    Device_Setting = 5
    EDID_Edit = 6
    Device_Other = 11
    IPC_Administration = 12
    Delete_Screen = 17
    Screen_3D = 18
    Screen_HDR = 19
    Screen_Genlock = 20
    Device_Check = 21
    IPC_Edit = 27
    Role_Operation = 30
    Edit_Stream = 32
    Open_Genlock = 33
    Preset_Effects = 34
    Device_Backups = 35
    User_Operation = 43


class DeviceConfig:
    Device_Font_Count = 30


class OutputCardMode:
    IndependentMode = 2 # 独立模式
    CopyMode = 1 # 复制模式
    BackupMode = 0 # 备份模式
    OutputTwoInOneCard = 3 # 二合一卡
    OutputTwoInOneCard_20 = 4


class IpcVideoMode:
    SplicingMode = 0
    FourWayMode = 1
    SixteenWayMode = 2
    EightWayMode = 3


class NdiVideoMode:
    SplicingMode = 0
    OneWayMode = 1
    TwoWayMode = 2

class NdiDecodeNumber:
    OneWayNumber = 1
    TwoWayNumber = 2
    FourWayNumber = 4

class IpcDecodeNumber:
    OneWayNumber = 1
    TwoWayNumber = 2
    FourWayNumber = 4
    EightWayNumber = 8
    SixteenWayNumber = 16


class IPCSlotId:
    Invalid = 255


class IPCTemplate:
    template = ["\"填写说明：\n",
                "1. 请使用模板文件进行填写并确保文件未加密，不要删减本说明及标题栏，带 * 为必填项，不能为空；\n"
                "2. 非必填字段(不带 * )中：RTSP协议数据需要填写厂商(大华、海康、宇视、雄迈、中维世纪、天地伟业、巨峰、尚维、天视通、技威、YOOSEE或其他)、端口、主码流、辅码流；\n",
                "3. 信号源名称：长度25个字符以内，通道名称：长度15个字符以内；\n",
                "4. 规格字段只能填写：200W、400W、800W；类型字段只能填写：IPC、NVR；配置方式只能填写：ONVIF、RTSP；\"\n",
                "* 信号源名称,* 规格,* 类型,* 配置方式,厂商,用户名,密码,* IP地址,端口号,* 通道名称,主码流,辅码流\n"]


class IPCExportConst:
    Decode_Capacity = {
        1: "800W",
        2: "200W",
        3: "400W"
    }
    IPC_Type = {
        0: "IPC",
        1: "NVR"
    }
    Source_Type = {
        0: "GB28181",
        1: "RTSP",
        2: "ONVIF"
    }
    Manufacturer = {
        -1: {
            1: "",
            0: ""
        },
        0: {
            1: "大华",
            0: "Dahua"
        },
        1: {
            1: "海康",
            0: "Hikvision"
        },
        2: {
            1: "宇视",
            0: "Uniview"
        },
        3: {
            1: "雄迈",
            0: "Xiongmai"
        },
        4: {
            1: "中维世纪",
            0: "Jovision"
        },
        5: {
            1: "天地伟业",
            0: "Tiandy"
        },
        6: {
            1: "巨峰",
            0: "Jufeng"
        },
        7: {
            1: "尚维",
            0: "Sunywo"
        },
        8: {
            1: "天视通",
            0: "Topsee"
        },
        9: {
            1: "技威",
            0: "Gwell"
        },
        10: {
            1: "YOOSEE",
            0: "YOOSEE"
        },
        255: {
            1: "其它",
            0: "Other"
        }
    }


class IPCImportConst:
    ONVIF = {
        "decodeCapacity": 0,
        "endIp": "",
        "ipList": [],
        "sourceName": "",
        "createTime": "",
        "ipcType": 0,
        "ip": "",
        "channelList": [
            {
                "channelIndex": 1,
                "channelName": "",
                "streamList": [
                    {
                        "streamIndex": 0,
                        "protocol": {
                            "type": 2,
                            "onvif": {
                                "mode": 0,
                                "onvifType": 0,
                                "onvifIp": "0",
                                "onvifUsrName": "",
                                "onvifPassWord": ""
                            }
                        }
                    },
                    {
                        "streamIndex": 1,
                        "protocol": {
                            "type": 2,
                            "onvif": {
                                "mode": 0,
                                "onvifType": 0,
                                "onvifIp": "0",
                                "onvifUsrName": "",
                                "onvifPassWord": ""
                            }
                        }
                    }
                ]
            }
        ],
        "deviceId": 0
    }
    RTSP = {
        "decodeCapacity": 0,
        "endIp": "",
        "ipList": [],
        "sourceName": "",
        "createTime": "",
        "ipcType": 0,
        "ip": "",
        "channelList": [
            {
                "channelIndex": 1,
                "channelName": "",
                "streamList": [
                    {
                        "streamIndex": 0,
                        "protocol": {
                            "type": 1,
                            "rtsp": {
                                "manufacturer": 255,
                                "rtspUrl": "",
                                "rtspIp": "",
                                "rtspPort": 0,
                                "rtspUsrName": "",
                                "rtspPassWord": "",
                                "mode": 0
                            }
                        }
                    },
                    {
                        "streamIndex": 1,
                        "protocol": {
                            "type": 1,
                            "rtsp": {
                                "manufacturer": 255,
                                "rtspUrl": "",
                                "rtspIp": "",
                                "rtspPort": 0,
                                "rtspUsrName": "",
                                "rtspPassWord": "",
                                "mode": 0
                            }
                        }
                    }
                ]
            }
        ],
        "deviceId": 0
    }
    Decode_Capacity = {
        "800W": 1,
        "200W": 2,
        "400W": 3
    }
    IPC_Type = {
        "IPC": 0,
        "NVR": 1
    }
    Source_Type = {
        "RTSP": 1,
        "ONVIF": 2
    }
    Manufacturer = {
        "大华": 0,
        "DAHUA": 0,
        "海康": 1,
        "HIKVISION": 1,
        "宇视": 2,
        "UNIVIEW": 2,
        "雄迈": 3,
        "XIONGMAI": 3,
        "中维世纪": 4,
        "JOVISION": 4,
        "天地伟业": 5,
        "Tiandy": 5,
        "巨峰": 6,
        "Jufeng": 6,
        "尚维": 7,
        "Sunywo": 7,
        "天视通": 8,
        "Topsee": 8,
        "技威": 9,
        "Gwell": 9,
        "YOOSEE": 10,
        "其它": 255,
        "OTHER": 255
    }
    Header_Count = 6

    Encoding_List = [None, "UTF-8", "GBK", "GB2312"]  # 读取导入文件的解码方式


class IPCType:
    IPC = 0
    NVR = 1

class IPCCardType:
    HS = 0
    RK = 1


class IPCSourceType:
    GB28181 = 0
    RTSP = 1
    ONVIF = 2


class IPCImportStatus:
    Importing = 0  # 导入中
    All_Success = 1  # 全部成功
    Part_Success = 2  # 部分成功


class IPCCheckRowConst:
    Success = ""
    Source_Name_Err = "信号源名称不合法"
    Decode_Capacity_Err = "规格不合法"
    IPC_Type_Err = "类型不合法"
    Source_Type_Err = "配置方式不合法"
    Manufacturer_Err = "厂商不合法"
    IP_Err = "ip不合法"
    Port_Err = "端口不合法"
    Main_URL_ERR = "主码流为空"
    Sub_URL_ERR = "辅码流为空"
    Channel_Name_Err = "通道名称不合法"
    Create_Source_Err = "创建失败"

    Source_Name_Length = 25
    Channel_Name_Length = 15

class outputModeType:
    Standard_Type = 1
    Customer_Type = 2


class SecurityMode:
    Normal = 1
    Safe = 2
    Default = Normal


class SecurityConfigType:
    SecurityModeConfig = 'securityModeConfig'
    LoginConfig = 'loginConfig'
    PwdConfig = 'pwdConfig'


class SecuritySessionExpiredType:
    FixedTime = 1
    UnfixedTime = 2
    UnOverdue = 3


NormalModeSettings = {
    # 登录配置
    'loginConfig': {
        'multiLoginAllow': 1,  # 同一账号多端登录: 0 不允许，1 允许
        'sessionExpireType': SecuritySessionExpiredType.UnOverdue,  # 1 固定过期， 2 滑动过期
        'sessionExpireTime': 30,  # 分钟
        'loginFailedCheck': {  # 登录失败校验
            'enable': 0,  # 是否启用校验 1 是,0 否
            'duration': 60,  # 登录校验周期(分钟)
            'showVerifyCodeTimes': 3,  # 失败N次出现验证码
            'lockUserTimes': 8,  # 失败N次锁定用户
        }
    },
    # 密码配置
    'pwdConfig': {
        'pwdLength': {  # 密码长度限制[5-50]
            'min': 5,
            'max': 20
        },
        'pwdRule': {  # 密码规则
            'nums': 0,  # 必须包含数字 1是,0否
            'lowerLetter': 0,  # 必须包含小写字母 1是,0否
            'upperLetter': 0,  # 必须包含大写 1是,0否
            'symbolCharacter': 0  # 必须包含特殊符号 1是,0否
        }

    }
}


SafeModeSettings = {
    # 登录配置
    'loginConfig': {
        'multiLoginAllow': 0,  # 同一账号多端登录: 0 不允许，1 允许
        'sessionExpireType': SecuritySessionExpiredType.UnfixedTime,  # 1 固定过期， 2 滑动过期
        'sessionExpireTime': 30,  # 分钟
        'loginFailedCheck': {  # 登录失败校验
            'enable': 1,  # 是否启用校验 1 是,0 否
            'duration': 60,  # 登录校验周期(分钟)
            'showVerifyCodeTimes': 2,  # 失败N次出现验证码
            'lockUserTimes': 7,  # 失败N次锁定用户
        }
    },
    # 密码配置
    'pwdConfig': {
        'pwdLength': {  # 密码长度限制[5-50]
            'min': 8,
            'max': 20
        },
        'pwdRule': {  # 密码规则
            'nums': 1,  # 必须包含数字 1是,0否
            'lowerLetter': 1,  # 必须包含小写字母 1是,0否
            'upperLetter': 1,  # 必须包含大写 1是,0否
            'symbolCharacter': 1  # 必须包含特殊符号 1是,0否
        }

    }
}
class UserStatus:
    """
    用户状态：活动、锁定、改密码
    """
    Active = 1
    Locked = 0
    PasswordChanged = 2


class Encrypt:
    Method = 'pbkdf2:sha256:100000'
    Salt_Len = 16


SN_Prefix = 'splicer'
Auth_Code_Expire_Time = 15 * 60
Audio_Support_Slot_ID = 7
Preset_Max_Limit = 256

class SafeConfig:
    IsNotSafe = 0
    IsSafe = 1

class HttpConfigType:
    DeviceHttpConfig = 'http_enable'
    Https_WebSocket_Port = 8081
    Http_WebSocket_Port = 8080

class UserLoginVerify:
    """用户登录验证"""
    UserActiveTime = 30*60
    VerifyCodeCount = 3

class LogConfig:
    Success = "operate success"

class DateConfig:
    FormatDate = '%Y-%m-%d %H:%M:%S'

class UserRoleType:
    Admin_Role = 1

class ReadSign:
    Is_ALL = 1
    Is_Not_ALL = 0

class Output12GSDIConst:
    LimitWidth = 3840
    LimitHeight = 2160
    LimitRefresh = 23


class PlanarConst:
    Bkg_And_Osd_Max_Count = {
        0x7525: 5,  # H_9U
        0x7524: 3,  # H_Ctrl_5U
        0x7523: 2,  # H_Ctrl_2U
        0x752C: 10,  # H_Ctrl_15U
        0x752D: 10,  # H_Ctrl_15U_PRO
    }

class PackagePwdConst:
    # 升级包密钥
    Secret_Key = "ptkLqneij80B8mVjdHYO15ioyHSxADPl"

    GCM_IV_Length = 12
    GCM_Tag_Length = 16


class DeviceInfoType:
    PackagePwdType = 'main_packagePwd'


if __name__ == '__main__':
    print(LayerType.Default)
